PC Base Station
Copyright (C) 2007 Joseph Bosman, Steve Olivieri, Ipek Ozil, Brandon Steacy

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA


#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <Windows.h>

#ifndef EM250_H
#define EM250_H

/* Return codes for main() */
#define MAIN_SUCCESS 0
#define MAIN_INPUT_ERROR 1
#define MAIN_SERIAL_PORT_FAIL 2
#define MAIN_LOG_FILE_FAIL 3
#define MAIN_OUTPUT_FILE_FAIL 4
#define MAIN_DATA_WRITE_ERROR 5
#define MAIN_READ_ERROR 6
#define MAIN_JOIN_FAIL 7
#define MAIN_COO_ERROR 8
#define MAIN_NO_CONNECTION 9
#define MAIN_REGISTER_FAIL 10

/* Return codes for compareTimeStructs() */
#define GREATER_THAN 1
#define LESS_THAN -1
#define EQUAL 0

/* Generic return codes for functions. */
#define SUCCESS 0
#define FAILURE 1

/* Sets the year, month, day, hour, minute, and second members of a SYSTIME
 * struct to 0.
 *	t: pointer to the SYSTIME to initialize
 */
void initTimeStruct(LPSYSTEMTIME t);

/* Sets two SYSTEMTIME structs equal to each other.  The first is filled with
 * the values from the second.
 *	t1: pointer to the SYSTEMTIME to be set
 *	t2: pointer to the SYSTEMTIME to get t1's new values from
 */
void setEqualTimeStructs(LPSYSTEMTIME t1, LPSYSTEMTIME t2);

/* Compares two SYSTIME structs to determine which has a more recent time.
 * If the first has a more recent time, GREATER_THAN is return.  If the second,
 * LESS_THAN.  If the two have the same time, return EQUAL.
 *	t1: pointer to the first SYSTIME to compare
 *	t2: pointer to the second SYSTIME to compare
 */
int compareTimeStructs(LPSYSTEMTIME t1, LPSYSTEMTIME t2);
 
/* Set up a COM port for communication.  This includes setting the baud rate, number
 * of bits and stop bits, etc.  We also set timeout values here and purge the input/output
 * buffers.
 *	port: the COM port to use, as a string (eg. COM1:)
 */
HANDLE setupCOM(LPCSTR port);

/* Writes data to the serial port.  Data should be formatted as a standard,
 * null-terminated string.  No LF or CR characters are necessary at the end.
 * If successful, just return 0.  Else, return non-zero.
 *	data: the data to send
 *	comPort: a pointer to the file handle for the serial port to write to
 *	len: the length of the data buffer
 *	command: true if this is an AT command, else false
 */
int writeSerial(LPCSTR data, PHANDLE comPort, UINT len, BOOL command);

/* Reads data from a file and stores it in the buffer pointed to by buff.
 * The value return is the number of bytes read.
 *	in: the file to read from
 *	buff: the buffer to store the read data in
 */
int readMessage(PHANDLE in, LPSTR buff);

/* Formats a message received from another ZigBee node so that it's more
 * useful to us.  The message is split into three buffers.  One contains the
 * actual message, another the hardware ID of the sender, and the third
 * contains both.  This function supports readable data (strings) as well
 * as binary data.  If binary data, binary should be 1, else 0.  This function
 * returns the length of the actual message (tBuff).
 *	tBuff: buffer to place the actual message in
 *	buff: buffer to place tBuff and hwID in
 *	hwID: buffer to place the hardware ID of the sender in
 *	binary: are we looking at binary data or readable data?
 *	l_tBuff: length of tBuff
 *	l_buff: length of buff
 *	l_hwID: length of hwID
 */
int formatMessage(LPSTR tBuff, LPSTR buff, LPSTR hwID, INT binary, INT l_tBuff, INT l_buff, INT l_hwID);

/* Reads the input buffer of the COM port until it finds either the desired
 * message or an "ERROR:" string.  If either is found, a pointer to the first
 * character in the found message is returned.  This function will block until
 * either the desired message or "ERROR:" is found, so use it with caution.
 *	in: pointer to the handle of the input file (COM port)
 *	message:	the message to search for
 *	out:	the file to log all read data to (can be NULL)
 */
LPSTR getMessage(PHANDLE in, LPCSTR message, PHANDLE out);

/* Returns the ceiling of x.  This function exists because for some reason I
 * can't figure out how to use a built-in function in Windows.
 *	x: the number to get the ceiling of
 */
DWORD ceiling(double x);

#endif /* EM250_H */
